@@ -48,6 +48,8 @@ module GitHash
   , giCommitCount
   , giCommitMessage
   , giDescribe
+  , giTag
+  , giFiles
     -- * Creators
   , getGitInfo
   , getGitRoot
@@ -62,8 +64,10 @@ import Control.Exception
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as B8
 import Data.Typeable (Typeable)
+import Data.Word (Word8)
 import Language.Haskell.TH
 import Language.Haskell.TH.Syntax
+import Language.Haskell.TH.Syntax.Compat
 import System.Directory
 import System.Exit
 import System.FilePath
@@ -83,6 +87,7 @@ data GitInfo = GitInfo
   , _giFiles :: ![FilePath]
   , _giCommitMessage :: !String
   , _giDescribe :: !String
+  , _giTag :: !String
   }
   deriving (Lift, Show)
 
@@ -119,8 +124,37 @@ giCommitMessage = _giCommitMessage
 giDescribe :: GitInfo -> String
 giDescribe = _giDescribe
 
+-- | The output of @git describe --always --tags@ for the most recent commit.
+--
+-- @since 0.1.5.0
+giTag :: GitInfo -> String
+giTag = _giTag
+
+-- | The files used to determine whether recompilation is necessary in splices.
+--
+-- @since 0.1.7.0
+giFiles :: GitInfo -> [FilePath]
+giFiles = _giFiles
+
 -- | Get a list of files from within a @.git@ directory.
 getGitFilesRegular :: FilePath -> IO [FilePath]
+-- [Note: Current implementation's limitation]
+-- the current implementation doesn't work right if:
+-- 1. the current branch's name contains Non-ASCII character (due to @B8.unpack@),
+-- 2. the current branch is only in .git/packed-refs, or
+-- 3. the current branch is a symbolic ref to another reference.
+-- In these cases, the file with the name `ref` in the following
+-- code cannot be found in the filesystem (in the cases 1 & 2),
+-- or can be found but will not be updated on commit (in the case 3).
+-- As a result, if a module uses @tGitInfo@ as TH macro
+-- and the target git repo is in one of the conditions 1--3
+-- at the time of compilation, content-change-free commits will fail to
+-- trigger recompilation.
+--
+-- [Note: reftable]
+-- In the near future, the technology called reftable may replace the
+-- Git's reference management. This function's implementation does not
+-- work with reftable, and therefore will need to be updated.
 getGitFilesRegular git = do
   -- a lot of bookkeeping to record the right dependencies
   let hd         = git </> "HEAD"
@@ -135,12 +169,12 @@ getGitFilesRegular git = do
       Right hdRef -> do
         -- the HEAD file either contains the hash of a detached head
         -- or a pointer to the file that contains the hash of the head
-        case B.splitAt 5 hdRef of
+        case B.splitAt 5 $ B.takeWhile (not . isSmallASCIIControl) hdRef of
           -- pointer to ref
           ("ref: ", relRef) -> do
             let ref = git </> B8.unpack relRef
             refExists <- doesFileExist ref
-            return $ if refExists then [ref] else []
+            return $ if refExists then [hd,ref] else [hd]
           -- detached head
           _hash -> return [hd]
   -- add the index if it exists to set the dirty flag
@@ -153,6 +187,15 @@ getGitFilesRegular git = do
   let files3 = if packedExists then [packedRefs] else []
 
   return $ concat [files1, files2, files3]
+  where
+    -- This is to quickly strip newline characters
+    -- from the content of .git/HEAD.
+    -- Git references don't include ASCII control char bytes:
+    -- 0x00 -- 0x1F and 0x7F.
+    -- .git/HEAD may contain some ASCII control bytes LF (0xA) and
+    -- CR (0xD) before EOF, which should be ignored.
+    isSmallASCIIControl :: Word8 -> Bool
+    isSmallASCIIControl = (<0x20)
 
 -- | Get a list of dependent files from a @.git@ file representing a
 -- git-worktree.
@@ -210,7 +253,9 @@ getGitInfo root = try $ do
 
   _giCommitMessage <- run ["log", "-1", "--pretty=%B"]
 
-  _giDescribe <- run ["describe", "--always"]
+  _giDescribe <- run ["describe", "--always", "--long"]
+
+  _giTag <- run ["describe", "--always", "--tags"]
 
   return GitInfo {..}
 
@@ -246,8 +291,8 @@ instance Exception GitHashException
 -- directory. Compilation fails if no info is available.
 --
 -- @since 0.1.0.0
-tGitInfo :: FilePath -> Q (TExp GitInfo)
-tGitInfo fp = unsafeTExpCoerce $ do
+tGitInfo :: FilePath -> SpliceQ GitInfo
+tGitInfo fp = unsafeSpliceCoerce $ do
   gi <- runIO $
     getGitRoot fp >>=
     either throwIO return >>=
@@ -260,8 +305,8 @@ tGitInfo fp = unsafeTExpCoerce $ do
 -- directory.
 --
 -- @since 0.1.2.0
-tGitInfoTry :: FilePath -> Q (TExp (Either String GitInfo))
-tGitInfoTry fp = unsafeTExpCoerce $ do
+tGitInfoTry :: FilePath -> SpliceQ (Either String GitInfo)
+tGitInfoTry fp = unsafeSpliceCoerce $ do
   egi <- runIO $ do
     eroot <- getGitRoot fp
     case eroot of
@@ -280,12 +325,12 @@ tGitInfoTry fp = unsafeTExpCoerce $ do
 -- working directory.
 --
 -- @since 0.1.0.0
-tGitInfoCwd :: Q (TExp GitInfo)
+tGitInfoCwd :: SpliceQ GitInfo
 tGitInfoCwd = tGitInfo "."
 
 -- | Try to load up the 'GitInfo' value at compile time for the current
 -- working directory.
 --
 -- @since 0.1.2.0
-tGitInfoCwdTry :: Q (TExp (Either String GitInfo))
+tGitInfoCwdTry :: SpliceQ (Either String GitInfo)
 tGitInfoCwdTry = tGitInfoTry "."
